/*
 *	fileread.c (usermain)
 *	Buffered file reader
 */

#include <basic.h>
#include <t2ex/fs.h>
#include "fileread.h"

#define min(a, b) (((a) < (b)) ? (a) : (b))

EXPORT ER file_open(file_reader* fp, const char* file, int flags, mode_t mode)
{
	fp->fd = fs_open(file, flags, mode);
	if (fp->fd < E_OK) {
		return fp->fd;
	}

	fp->sp = 0;
	fp->ep = 0;
	fp->eof = FALSE;

	return E_OK;
}

EXPORT int file_read(file_reader* fp, void* buf, size_t count)
{
	B* b = (B*)buf;
	int pos = 0, sz;
	int er;

start_over:
	if (fp->sp < fp->ep) {
		/* 読込み済みのバッファからコピー */
		sz = min(fp->ep - fp->sp, count - pos);
		memcpy(b + pos, fp->buf + fp->sp, sz);

		fp->sp += sz;
		pos += sz;
	}
	while (pos < count && !fp->eof) {
		/* バッファの中身から足りない分については、サイズに応じて直接読込みまたはバッファ経由で読込み */
		fp->sp = fp->ep = 0;
		if (count - pos >= BUFFER_SIZE) {
			er = fs_read(fp->fd, b + pos, count - pos);
			if (er <= E_OK) {
				if (er == 0) fp->eof = TRUE;
				return pos;
			}
			pos += er;
		}
		else {
			er = fs_read(fp->fd, fp->buf, BUFFER_SIZE);
			if (er <= E_OK) {
				if (er == 0) fp->eof = TRUE;
				return pos;
			}
			fp->ep = er;
			goto start_over;
		}
	}
	return pos;
}

EXPORT ER file_close(file_reader* fp)
{
	return fs_close(fp->fd);
}
