/*
 *	usermain.c (usermain)
 *	User Main
 */

#include <basic.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <tk/tkernel.h>
#include <tm/tmonitor.h>

/* Device Drivers */
IMPORT ER ConsoleIO( INT ac, UB *av[] );
IMPORT ER ClockDrv( INT ac, UB *av[] );
IMPORT ER SysDiskDrv( INT ac, UB *av[] );
IMPORT ER ScreenDrv( INT ac, UB *av[] );

/* T2EX extension modules */
IMPORT ER fs_main( INT ac, UB* av[] );

/* Application program */
IMPORT void appl_main(void);

/*
 * Entry routine for the user application.
 * At this point, Initialize and start the user application.
 *
 * Entry routine is called from the initial task for Kernel,
 * so system call for stopping the task should not be issued
 * from the contexts of entry routine.
 * We recommend that:
 * (1)'usermain()' only generates the user initial task.
 * (2)initialize and start the user application by the user
 * initial task.
 */
EXPORT	INT	usermain( void )
{
	ER	err;

	/* start the device driver */
	err = ConsoleIO(0, NULL);
	tm_putstring(err >= E_OK ? "ConsoleIO - OK\n" : "ConsoleIO - ERR\n");

	err = ClockDrv(0, NULL);
	tm_putstring(err >= E_OK ? "ClockDrv - OK\n" : "ClockDrv - ERR\n");

	err = SysDiskDrv(0, NULL);
	tm_putstring(err >= E_OK ? "SysDiskDrv - OK\n" : "SysDiskDrv - ERR\n");

	err = ScreenDrv(0, NULL);
	tm_putstring(err >= E_OK ? "ScreenDrv - OK\n" : "ScreenDrv - ERR\n");

	err = fs_main(0, NULL);
	tm_putstring(err >= E_OK ? "fs_main(0) - OK\n" : "fs_main(0) - ERR\n");

	/* start the T2EX application */
	tm_putstring("*** T2EX Application program start !!\n");
	appl_main();

	/* shutdowm the T2EX extension modules */
	err = fs_main(-1, NULL);
	tm_putstring(err >= E_OK ? "fs_main(-1) - OK\n":"fs_main(-1) - ERR\n");

	tm_putstring((UB*)"Push 'g' key to shutdown the T-Kernel.\n");
	tm_monitor();

	return 0;
}

